#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import sys
import numpy as np
import math

import rospy
import rospkg

from std_msgs.msg import Header
from visualization_msgs.msg import MarkerArray, Marker
from sensor_msgs.msg import PointCloud2, PointField
import sensor_msgs.point_cloud2 as pc2


def coords2pc2msg(coords, sensing_radius):

    n = len(coords)

    data = []
    for i in range(n):
        x = float(coords[i][0])
        y = float(coords[i][1])
        r = sensing_radius

        pt = [x,y,r]
        data.append(pt)
    
    fields = [PointField('x', 0, PointField.FLOAT32, 1),
              PointField('y', 4, PointField.FLOAT32, 1),
              PointField('radius', 8, PointField.FLOAT32, 1)
              ]

    header = Header()
    msg = pc2.create_cloud(header, fields, data)

    return msg

def coords2MarkerArrayMsg(coords, sensing_radius, color=[255,155,0,50]):
  
    msg = MarkerArray()

    marker_i = -1

    for coord in coords:

        marker_i += 1

        marker = Marker()

        marker.header.stamp = rospy.Time.now()
        marker.header.frame_id = 'map'
        marker.ns = 'goals'
        marker.id = marker_i

        marker.type = 2 # sphere
        marker.action = 0 # add
        marker.lifetime = rospy.Duration(0)

        marker.pose.position.x = coord[0]
        marker.pose.position.y = coord[1]
        marker.pose.position.z = 0.1

        marker.scale.x = sensing_radius*2
        marker.scale.y = sensing_radius*2
        marker.scale.z = .1

        marker.color.r = float(color[0])/255
        marker.color.g = float(color[1])/255
        marker.color.b = float(color[2])/255
        marker.color.a = float(color[3])/100

        msg.markers.append(marker)

    return msg

if __name__ == '__main__':

    # init ros node
    rospy.init_node('uir_goals_publisher', log_level=rospy.DEBUG)
    rospy.loginfo("Starting UIR goals publisher planner")

    rospack = rospkg.RosPack()
   
    # load parameters
    problem_path = rospy.get_param("~PROBLEM_PATH",rospack.get_path('uir_multi_goal_tools') + "/problems/att48.txt")
    goals_topic_name = rospy.get_param("~GOALS_TOPIC_NAME","/goals")
    goals_nh_topic_name = rospy.get_param("~GOALS_NEIGHBORHOODS_TOPIC_NAME","/goals/neighborhoods")
    sensing_radius = float(rospy.get_param("~SENSING_RADIUS",0.5))

    goals_publisher = rospy.Publisher(goals_topic_name, PointCloud2, queue_size=1)
    goals_nh_publisher = rospy.Publisher(goals_nh_topic_name, MarkerArray, queue_size=1)

    goals = np.loadtxt(problem_path)[:,1:]

    msg_ma = coords2MarkerArrayMsg([goals[i,:] for i in range(goals.shape[0])],sensing_radius)
    msg_pc2 = coords2pc2msg([goals[i,:] for i in range(goals.shape[0])],sensing_radius)

    goals_publisher.publish(msg_pc2)
    goals_nh_publisher.publish(msg_ma)

    # report loop
    rate = rospy.Rate(1)
    while not rospy.is_shutdown():
        rate.sleep()
        goals_publisher.publish(msg_pc2)
        goals_nh_publisher.publish(msg_ma)
        rospy.loginfo("%s : I am alive" % rospy.get_name())
    

